@@ -6,6 +6,9 @@ import xml.etree.ElementTree as ET
 #-------------------------------------#
 import datetime
 import os
+import cv2
+import time
+from tqdm import tqdm
 
 import numpy as np
 import torch
@@ -22,6 +25,12 @@ from .utils.callbacks import EvalCallback, LossHistory
 from .utils.dataloader import YoloDataset, yolo_dataset_collate
 from .utils.utils import download_weights, get_anchors, get_classes, show_config
 from .utils.utils_fit import fit_one_epoch
+from .utils.utils_map import get_coco_map, get_map
+
+
+from .yolo import YOLO
+from PIL import Image
+
 
 
 
@@ -29,13 +38,20 @@ from .utils.utils_fit import fit_one_epoch
 class Yolov7:
 
     
-    def __init__(self, classes_path: str, VOCdevkit_path: str = 'VOCdevkit' ) -> None:
+    def __init__(self, classes_path: str, VOCdevkit_path: str = os.path.join('yolovxx', 'yolov7pytorch', 'VOCdevkit'),
+                 anchors_path: str = os.path.join('yolov7pytorch', 'model_data', 'yolo_anchors.txt'),
+                 phi: str = "l", input_shape: list = [640, 640], anchors_mask: list = [[6, 7, 8], [3, 4, 5], [0, 1, 2]]) -> None:
+        
         self.classes_path = classes_path
         self.VOCdevkit_path = VOCdevkit_path
+        self.anchors_path = anchors_path
         self.classes, _  = get_classes(self.classes_path)
         self.VOCdevkit_sets: list  = [('2007', 'train'), ('2007', 'val')]
         self.photo_nums  = np.zeros(len(self.VOCdevkit_sets))
         self.nums = np.zeros(len(self.classes))
+        self.input_shape = input_shape
+        self.phi = phi
+        self.anchors_mask = anchors_mask
 
 
 
@@ -77,8 +93,8 @@ class Yolov7:
 
         if annotation_mode == 0 or annotation_mode == 1:
             print("Generate txt in ImageSets.")
-            xmlfilepath     = os.path.join(self.VOCdevkit_path, 'VOC2007/Annotations')
-            saveBasePath    = os.path.join(self.VOCdevkit_path, 'VOC2007/ImageSets/Main')
+            xmlfilepath     = os.path.join(self.VOCdevkit_path, 'VOC2007', 'Annotations')
+            saveBasePath    = os.path.join(self.VOCdevkit_path, 'VOC2007', 'ImageSets/Main')
             temp_xml        = os.listdir(xmlfilepath)
             total_xml       = []
             for xml in temp_xml:
@@ -163,12 +179,11 @@ class Yolov7:
     
     
     def train(self, cuda: bool = True, distributed: bool = False, sync_bn: bool = False,
-              fp16: bool = False, anchors_path: str = 'yolovxx/yolovxx/yolov7pytorch/model_data/yolo_anchors.txt', pretrained: bool = False,
-              model_path: str = "", phi: str = "l", input_shape: list = [640, 640],
-              save_dir: str = 'logs', save_period: int = 10, eval_flag: bool= True,
-              eval_period: int = 10, num_workers: int = 4) -> None:
+              fp16: bool = False, pretrained: bool = False, model_path: str = "", save_dir: str = 'logs',
+              save_period: int = 10, eval_flag: bool= True, eval_period: int = 10, num_workers: int = 4,
+              epoch: int = 300, optimizer_type: str = 'sgd', batch_size: int = 4, freeze_train: bool = True) -> None:
             
-        anchors_mask = [[6, 7, 8], [3, 4, 5], [0, 1, 2]]
+        anchors_mask = self.anchors_mask
 
 
         mosaic              = True
@@ -216,8 +231,8 @@ class Yolov7:
         #                       (当Freeze_Train=False时失效)
         #------------------------------------------------------------------#
         Init_Epoch          = 0
-        Freeze_Epoch        = 50
-        Freeze_batch_size   = 8
+        Freeze_Epoch        = epoch
+        Freeze_batch_size   = batch_size
         #------------------------------------------------------------------#
         #   解冻阶段训练参数
         #   此时模型的主干不被冻结了，特征提取网络会发生改变
@@ -227,13 +242,13 @@ class Yolov7:
         #                           Adam可以使用相对较小的UnFreeze_Epoch
         #   Unfreeze_batch_size     模型在解冻后的batch_size
         #------------------------------------------------------------------#
-        UnFreeze_Epoch      = 300
-        Unfreeze_batch_size = 4
+        UnFreeze_Epoch      = epoch
+        Unfreeze_batch_size = batch_size
         #------------------------------------------------------------------#
         #   Freeze_Train    是否进行冻结训练
         #                   默认先冻结主干训练后解冻训练。
         #------------------------------------------------------------------#
-        Freeze_Train        = True
+        Freeze_Train        = freeze_train
 
         #------------------------------------------------------------------#
         #   其它训练参数：学习率、优化器、学习率下降有关
@@ -252,7 +267,7 @@ class Yolov7:
         #   weight_decay    权值衰减，可防止过拟合
         #                   adam会导致weight_decay错误，使用adam时建议设置为0。
         #------------------------------------------------------------------#
-        optimizer_type      = "sgd"
+        optimizer_type      = optimizer_type
         momentum            = 0.937
         weight_decay        = 5e-4
         #------------------------------------------------------------------#
@@ -284,7 +299,7 @@ class Yolov7:
         #   获取classes和anchor
         #------------------------------------------------------#
         class_names, num_classes = get_classes(self.classes_path)
-        anchors, num_anchors     = get_anchors(anchors_path)
+        anchors, num_anchors     = get_anchors(self.anchors_path)
 
         #----------------------------------------------------#
         #   下载预训练权重
@@ -292,15 +307,15 @@ class Yolov7:
         if pretrained:
             if distributed:
                 if local_rank == 0:
-                    download_weights(phi)  
+                    download_weights(self.phi)  
                 dist.barrier()
             else:
-                download_weights(phi)
+                download_weights(self.phi)
                 
         #------------------------------------------------------#
         #   创建yolo模型
         #------------------------------------------------------#
-        model = YoloBody(anchors_mask, num_classes, phi, pretrained=pretrained)
+        model = YoloBody(anchors_mask, num_classes, self.phi, pretrained=pretrained)
         if not pretrained:
             weights_init(model)
         if model_path != '':
@@ -335,14 +350,14 @@ class Yolov7:
         #----------------------#
         #   获得损失函数
         #----------------------#
-        yolo_loss    = YOLOLoss(anchors, num_classes, input_shape, anchors_mask, label_smoothing)
+        yolo_loss    = YOLOLoss(anchors, num_classes, self.input_shape, anchors_mask, label_smoothing)
         #----------------------#
         #   记录Loss
         #----------------------#
         if local_rank == 0:
             time_str        = datetime.datetime.strftime(datetime.datetime.now(),'%Y_%m_%d_%H_%M_%S')
             log_dir         = os.path.join(save_dir, "loss_" + str(time_str))
-            loss_history    = LossHistory(log_dir, model, input_shape=input_shape)
+            loss_history    = LossHistory(log_dir, model, input_shape=self.input_shape)
         else:
             loss_history    = None
             
@@ -394,7 +409,7 @@ class Yolov7:
 
         if local_rank == 0:
             show_config(
-                classes_path = self.classes_path, anchors_path = anchors_path, anchors_mask = anchors_mask, model_path = model_path, input_shape = input_shape, \
+                classes_path = self.classes_path, anchors_path = self.anchors_path, anchors_mask = anchors_mask, model_path = model_path, input_shape = self.input_shape, \
                 Init_Epoch = Init_Epoch, Freeze_Epoch = Freeze_Epoch, UnFreeze_Epoch = UnFreeze_Epoch, Freeze_batch_size = Freeze_batch_size, Unfreeze_batch_size = Unfreeze_batch_size, Freeze_Train = Freeze_Train, \
                 Init_lr = Init_lr, Min_lr = Min_lr, optimizer_type = optimizer_type, momentum = momentum, lr_decay_type = lr_decay_type, \
                 save_period = save_period, save_dir = save_dir, num_workers = num_workers, num_train = num_train, num_val = num_val
@@ -484,9 +499,9 @@ class Yolov7:
             #---------------------------------------#
             #   构建数据集加载器。
             #---------------------------------------#
-            train_dataset   = YoloDataset(train_lines, input_shape, num_classes, anchors, anchors_mask, epoch_length=UnFreeze_Epoch, \
+            train_dataset   = YoloDataset(train_lines, self.input_shape, num_classes, anchors, anchors_mask, epoch_length=UnFreeze_Epoch, \
                                             mosaic=mosaic, mixup=mixup, mosaic_prob=mosaic_prob, mixup_prob=mixup_prob, train=True, special_aug_ratio=special_aug_ratio)
-            val_dataset     = YoloDataset(val_lines, input_shape, num_classes, anchors, anchors_mask, epoch_length=UnFreeze_Epoch, \
+            val_dataset     = YoloDataset(val_lines, self.input_shape, num_classes, anchors, anchors_mask, epoch_length=UnFreeze_Epoch, \
                                             mosaic=False, mixup=False, mosaic_prob=0, mixup_prob=0, train=False, special_aug_ratio=0)
             
             if distributed:
@@ -508,7 +523,7 @@ class Yolov7:
             #   记录eval的map曲线
             #----------------------#
             if local_rank == 0:
-                eval_callback   = EvalCallback(model, input_shape, anchors, anchors_mask, class_names, num_classes, val_lines, log_dir, cuda, \
+                eval_callback   = EvalCallback(model, self.input_shape, anchors, anchors_mask, class_names, num_classes, val_lines, log_dir, cuda, \
                                                 eval_flag=eval_flag, period=eval_period)
             else:
                 eval_callback   = None
@@ -573,17 +588,215 @@ class Yolov7:
                     dist.barrier()
 
 
-    def predict(self, cuda: bool = True, distributed: bool = False, sync_bn: bool = False,
-              fp16: bool = False, anchors_path: str = 'model_data/yolo_anchors.txt', pretrained: bool = False,
-              model_path: str = "", phi: str = "l", input_shape: list = [640, 640],
-              save_dir: str = 'logs', save_period: int = 10, eval_flag: bool= True,
-              eval_period: int = 10, num_workers: int = 4) -> None:
-        pass
+    def predict(self, cuda: bool = True, model_path: str = "", mode: str = "predict",
+                imagefile_path: str = "",
+                from_webcam: bool = False, webcam_num: int = 0, video_path: str = "",
+                video_save_path: str = ".", video_fps: float = 25.0, dir_origin_path: str = "",
+                dir_save_path: str = ".", save_heatmap_output_as: str = "heatmap_out", crop: bool = False,
+                count: bool = False,  test_interval: int = 100,
+                simplify: bool= True, onnx_save_path: str = ".", confidence: float = 0.5,
+                nms_iou: float = 0.3, letterbox_image: bool = True):
         
 
+        yolo = YOLO(model_path=model_path, classes_path=self.classes_path, anchors_path=self.anchors_path,
+                    anchors_mask=self.anchors_mask, input_shape=self.input_shape, phi=self.phi,
+                    confidence=confidence, nms_iou=nms_iou, letterbox_image=letterbox_image, cuda=cuda)
+        
+        if mode == "predict":
+            '''
+            1、如果想要进行检测完的图片的保存，利用r_image.save("img.jpg")即可保存，直接在predict.py里进行修改即可。 
+            2、如果想要获得预测框的坐标，可以进入yolo.detect_image函数，在绘图部分读取top，left，bottom，right这四个值。
+            3、如果想要利用预测框截取下目标，可以进入yolo.detect_image函数，在绘图部分利用获取到的top，left，bottom，right这四个值
+            在原图上利用矩阵的方式进行截取。
+            4、如果想要在预测图上写额外的字，比如检测到的特定目标的数量，可以进入yolo.detect_image函数，在绘图部分对predicted_class进行判断，
+            比如判断if predicted_class == 'car': 即可判断当前目标是否为车，然后记录数量即可。利用draw.text即可写字。
+            '''
+            # while True:
+                # img = input('Input image filename:')
+            try:
+                image = Image.open(imagefile_path)
+            except:
+                print('Open Error! Try again!')
+                return 
+            else:
+                r_image, detection_result = yolo.detect_image(image, crop = crop, count=count)
+                r_image.show()
+
+                return detection_result
+
+        elif mode == "video":
+            if from_webcam:
+                capture = cv2.VideoCapture(webcam_num)
+            else:
+                capture = cv2.VideoCapture(video_path)
+            
+            if video_save_path!="":
+                fourcc  = cv2.VideoWriter_fourcc(*'XVID')
+                size    = (int(capture.get(cv2.CAP_PROP_FRAME_WIDTH)), int(capture.get(cv2.CAP_PROP_FRAME_HEIGHT)))
+                out     = cv2.VideoWriter(video_save_path, fourcc, video_fps, size)
+
+            ref, frame = capture.read()
+            if not ref:
+                raise ValueError("未能正确读取摄像头（视频），请注意是否正确安装摄像头（是否正确填写视频路径）。")
+
+            fps = 0.0
+            while(True):
+                t1 = time.time()
+                # 读取某一帧
+                ref, frame = capture.read()
+                if not ref:
+                    break
+                # 格式转变，BGRtoRGB
+                frame = cv2.cvtColor(frame,cv2.COLOR_BGR2RGB)
+                # 转变成Image
+                frame = Image.fromarray(np.uint8(frame))
+                # 进行检测
+                frame, detection_result = yolo.detect_image(frame)
+                frame = np.array(frame)
+                # RGBtoBGR满足opencv显示格式
+                frame = cv2.cvtColor(frame,cv2.COLOR_RGB2BGR)
+                
+                fps  = ( fps + (1./(time.time()-t1)) ) / 2
+                print("fps= %.2f"%(fps))
+                frame = cv2.putText(frame, "fps= %.2f"%(fps), (0, 40), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
+                
+                cv2.imshow("video",frame)
+                c= cv2.waitKey(1) & 0xff 
+                if video_save_path!="":
+                    out.write(frame)
+
+                if c==27:
+                    capture.release()
+                    break
+
+            print("Video Detection Done!")
+            capture.release()
+            if video_save_path!="":
+                print("Save processed video to the path :" + video_save_path)
+                out.release()
+            cv2.destroyAllWindows()
+            
+        elif mode == "fps":
+            img = Image.open(imagefile_path)
+            tact_time = yolo.get_FPS(img, test_interval)
+            print(str(tact_time) + ' seconds, ' + str(1/tact_time) + 'FPS, @batch_size 1')
+
+        elif mode == "dir_predict":
+            import os
+
+            from tqdm import tqdm
+
+            img_names = os.listdir(dir_origin_path)
+            for img_name in tqdm(img_names):
+                if img_name.lower().endswith(('.bmp', '.dib', '.png', '.jpg', '.jpeg', '.pbm', '.pgm', '.ppm', '.tif', '.tiff')):
+                    image_path  = os.path.join(dir_origin_path, img_name)
+                    image       = Image.open(image_path)
+                    r_image, detection_result = yolo.detect_image(image)
+                    if not os.path.exists(dir_save_path):
+                        os.makedirs(dir_save_path)
+                    r_image.save(os.path.join(dir_save_path, img_name.replace(".jpg", ".png")), quality=95, subsampling=0)
+
+        elif mode == "heatmap":
+            while True:
+                img = input('Input image filename:')
+                try:
+                    image = Image.open(img)
+                except:
+                    print('Open Error! Try again!')
+                    continue
+                else:
+                    yolo.detect_heatmap(image, save_heatmap_output_as)
+                    
+        elif mode == "export_onnx":
+            yolo.convert_to_onnx(simplify, onnx_save_path)
+            
+        else:
+            raise AssertionError("Please specify the correct mode: 'predict', 'video', 'fps', 'heatmap', 'export_onnx', 'dir_predict'.")
+
+
+
+    def compute_map(self, model_path: str = '', cuda: bool = True, map_mode: int = 0, min_overlap: float = 0.5, confidence: float = 0.001,
+                    nms_iou: float = 0.3, score_threhold: float = 0.5, map_vis: bool = False,
+                    map_out_path: str = "map_out") -> None:
+
+        '''
+        Recall和Precision不像AP是一个面积的概念，因此在门限值（Confidence）不同时，网络的Recall和Precision值是不同的。
+        默认情况下，本代码计算的Recall和Precision代表的是当门限值（Confidence）为0.5时，所对应的Recall和Precision值。
+
+        受到mAP计算原理的限制，网络在计算mAP时需要获得近乎所有的预测框，这样才可以计算不同门限条件下的Recall和Precision值
+        因此，本代码获得的map_out/detection-results/里面的txt的框的数量一般会比直接predict多一些，目的是列出所有可能的预测框，
+        '''
+
+        image_ids = open(os.path.join(self.VOCdevkit_path, "VOC2007/ImageSets/Main/test.txt")).read().strip().split()
+
+        if not os.path.exists(map_out_path):
+            os.makedirs(map_out_path)
+        if not os.path.exists(os.path.join(map_out_path, 'ground-truth')):
+            os.makedirs(os.path.join(map_out_path, 'ground-truth'))
+        if not os.path.exists(os.path.join(map_out_path, 'detection-results')):
+            os.makedirs(os.path.join(map_out_path, 'detection-results'))
+        if not os.path.exists(os.path.join(map_out_path, 'images-optional')):
+            os.makedirs(os.path.join(map_out_path, 'images-optional'))
+
+        class_names, _ = get_classes(self.classes_path)
+
+        if map_mode == 0 or map_mode == 1:
+            print("Load model.")
+            yolo = YOLO(model_path=model_path, classes_path=self.classes_path, anchors_path=self.anchors_path,
+                    anchors_mask=self.anchors_mask, input_shape=self.input_shape, phi=self.phi,
+                    confidence=confidence, nms_iou=nms_iou, cuda=cuda)
+            print("Load model done.")
+
+            print("Get predict result.")
+            for image_id in tqdm(image_ids):
+                image_path  = os.path.join(self.VOCdevkit_path, "VOC2007/JPEGImages/"+image_id+".jpg")
+                image       = Image.open(image_path)
+                if map_vis:
+                    image.save(os.path.join(map_out_path, "images-optional/" + image_id + ".jpg"))
+                yolo.get_map_txt(image_id, image, class_names, map_out_path)
+            print("Get predict result done.")
+            
+        if map_mode == 0 or map_mode == 2:
+            print("Get ground truth result.")
+            for image_id in tqdm(image_ids):
+                with open(os.path.join(map_out_path, "ground-truth/"+image_id+".txt"), "w") as new_f:
+                    root = ET.parse(os.path.join(self.VOCdevkit_path, "VOC2007/Annotations/"+image_id+".xml")).getroot()
+                    for obj in root.findall('object'):
+                        difficult_flag = False
+                        if obj.find('difficult')!=None:
+                            difficult = obj.find('difficult').text
+                            if int(difficult)==1:
+                                difficult_flag = True
+                        obj_name = obj.find('name').text
+                        if obj_name not in class_names:
+                            continue
+                        bndbox  = obj.find('bndbox')
+                        left    = bndbox.find('xmin').text
+                        top     = bndbox.find('ymin').text
+                        right   = bndbox.find('xmax').text
+                        bottom  = bndbox.find('ymax').text
+
+                        if difficult_flag:
+                            new_f.write("%s %s %s %s %s difficult\n" % (obj_name, left, top, right, bottom))
+                        else:
+                            new_f.write("%s %s %s %s %s\n" % (obj_name, left, top, right, bottom))
+            print("Get ground truth result done.")
+
+        if map_mode == 0 or map_mode == 3:
+            print("Get map.")
+            get_map(min_overlap, True, score_threhold = score_threhold, path = map_out_path)
+            print("Get map done.")
+
+        if map_mode == 4:
+            print("Get map.")
+            get_coco_map(class_names = class_names, path = map_out_path)
+            print("Get map done.")
+
+
+
 
 def main():
-    yolo = Yolov7(classes_path="yolovxx/yolov7pytorch/model_data/voc_classes.txt")
+    yolo = Yolov7(classes_path=os.path.join('yolovxx', 'yolov7pytorch', 'model_data', 'voc_classes.txt'))
     yolo.voc_annotation()
     yolo.train()
 
