@@ -1,16 +1,21 @@
 import colorsys
 import os
 import time
+import cv2
+import matplotlib
+matplotlib.use('TkAgg')
+import matplotlib.pyplot as plt
+        
 
 import numpy as np
 import torch
 import torch.nn as nn
 from PIL import ImageDraw, ImageFont
 
-from nets.yolo import YoloBody
-from utils.utils import (cvtColor, get_anchors, get_classes, preprocess_input,
+from .nets.yolo import YoloBody
+from .utils.utils import (cvtColor, get_anchors, get_classes, preprocess_input,
                          resize_image, show_config)
-from utils.utils_bbox import DecodeBox
+from .utils.utils_bbox import DecodeBox
 
 '''
 训练自己的数据集必看注释！
@@ -160,7 +165,7 @@ class YOLO(object):
         #---------------------------------------------------------#
         #   设置字体与边框厚度
         #---------------------------------------------------------#
-        font        = ImageFont.truetype(font='model_data/simhei.ttf', size=np.floor(3e-2 * image.size[1] + 0.5).astype('int32'))
+        font        = ImageFont.truetype(font='yolovxx/yolov7pytorch/model_data/simhei.ttf', size=np.floor(3e-2 * image.size[1] + 0.5).astype('int32'))
         thickness   = int(max((image.size[0] + image.size[1]) // np.mean(self.input_shape), 1))
         #---------------------------------------------------------#
         #   计数
@@ -194,6 +199,7 @@ class YOLO(object):
         #---------------------------------------------------------#
         #   图像绘制
         #---------------------------------------------------------#
+        detection_result = {}
         for i, c in list(enumerate(top_label)):
             predicted_class = self.class_names[int(c)]
             box             = top_boxes[i]
@@ -206,6 +212,8 @@ class YOLO(object):
             bottom  = min(image.size[1], np.floor(bottom).astype('int32'))
             right   = min(image.size[0], np.floor(right).astype('int32'))
 
+            detection_result[i] = {"predicted_class": predicted_class, "bboxes": [top, left, bottom, right], "score": score}
+
             label = '{} {:.2f}'.format(predicted_class, score)
             draw = ImageDraw.Draw(image)
             label_size = draw.textsize(label, font)
@@ -223,7 +231,7 @@ class YOLO(object):
             draw.text(text_origin, str(label,'UTF-8'), fill=(0, 0, 0), font=font)
             del draw
 
-        return image
+        return image, detection_result
 
     def get_FPS(self, image, test_interval):
         image_shape = np.array(np.shape(image)[0:2])
@@ -275,9 +283,8 @@ class YOLO(object):
         tact_time = (t2 - t1) / test_interval
         return tact_time
 
-    def detect_heatmap(self, image, heatmap_save_path):
-        import cv2
-        import matplotlib.pyplot as plt
+    def detect_heatmap(self, image, save_heatmap_output_as):
+
         def sigmoid(x):
             y = 1.0 / (1.0 + np.exp(-x))
             return y
@@ -322,8 +329,8 @@ class YOLO(object):
         plt.axis('off')
         plt.subplots_adjust(top=1, bottom=0, right=1,  left=0, hspace=0, wspace=0)
         plt.margins(0, 0)
-        plt.savefig(heatmap_save_path, dpi=200, bbox_inches='tight', pad_inches = -0.1)
-        print("Save to the " + heatmap_save_path)
+        plt.savefig(save_heatmap_output_as, dpi=200, bbox_inches='tight', pad_inches = -0.1)
+        print("Heatmap figure saved as " + save_heatmap_output_as + '.png')
         plt.show()
 
     def convert_to_onnx(self, simplify, model_path):
